from typing import Type

from django.conf import settings
from django.contrib import admin
from django.contrib.admin import SimpleListFilter
from django.db.models import QuerySet
from django.template.loader import render_to_string
from edc_model_admin.dashboard import ModelAdminDashboardMixin
from edc_model_admin.mixins import (
    ModelAdminFormInstructionsMixin,
    TemplatesModelAdminMixin,
)
from edc_qareports.modeladmin_mixins import QaReportModelAdminMixin
from edc_sites.admin import SiteModelAdminMixin
from edc_sites.admin.list_filters import SiteListFilter
from edc_visit_schedule.admin import ScheduleStatusListFilter

from ..admin_site import meta_reports_admin
from ..models import Endpoints
from ..tasks import update_unmanaged_table


def update_unmanaged_table_action(modeladmin, request, queryset):
    subject_identifiers = []
    if queryset.count() != modeladmin.model.objects.count():
        subject_identifiers = [o.subject_identifier for o in queryset]
    if settings.CELERY_ENABLED:
        return update_unmanaged_table.delay(subject_identifiers)
    return update_unmanaged_table(subject_identifiers)


update_unmanaged_table_action.short_description = "Regenerate report for selected subjects"


@admin.register(Endpoints, site=meta_reports_admin)
class EndpointAdmin(
    QaReportModelAdminMixin,
    SiteModelAdminMixin,
    ModelAdminDashboardMixin,
    ModelAdminFormInstructionsMixin,
    TemplatesModelAdminMixin,
    admin.ModelAdmin,
):

    change_list_note = render_to_string("meta_reports/endpoints_changelist_note.html")
    actions = [update_unmanaged_table_action]
    qa_report_list_display_insert_pos = 2
    ordering = ["-fbg_datetime"]
    list_display = [
        "dashboard",
        "subject",
        "visit",
        "fbg_date",
        "fast",
        "fbg",
        "ogtt",
        "endpoint",
        "last_updated",
        "offstudy_datetime",
        "offstudy_reason",
    ]

    list_filter = [
        "endpoint_label",
        ScheduleStatusListFilter,
        SiteListFilter,
    ]

    search_fields = ["subject_identifier"]

    def get_queryset(self, request) -> QuerySet:
        qs = super().get_queryset(request)
        qs = qs.filter(offstudy_datetime__isnull=True)
        return qs

    def get_list_filter(self, request) -> tuple[str | Type[SimpleListFilter], ...]:
        list_filter = super().get_list_filter(request)
        list_filter = list_filter + (SiteListFilter,)
        return list_filter

    @admin.display(description="subject", ordering="subject_identifier")
    def subject(self, obj=None):
        return obj.subject_identifier

    @admin.display(description="visit", ordering="visit_code")
    def visit(self, obj=None):
        return obj.visit_code

    @admin.display(description="FBG DATE", ordering="fbg_datetime")
    def fbg_date(self, obj=None):
        return obj.fbg_datetime.date() if obj.fbg_datetime else None

    @admin.display(description="FAST", ordering="fasting")
    def fast(self, obj=None):
        return obj.fasting

    @admin.display(description="FBG", ordering="fbg_value")
    def fbg(self, obj=None):
        return obj.fbg_value

    @admin.display(description="OGTT", ordering="ogtt_value")
    def ogtt(self, obj=None):
        return obj.ogtt_value

    @admin.display(description="endpoint", ordering="endpoint_label")
    def endpoint(self, obj=None):
        return obj.endpoint_label

    @admin.display(description="last_updated", ordering="created")
    def last_updated(self, obj=None):
        return obj.created
